#include <allegro.h>
//#include <crtdbg.h>

#define RAND_VEHICLE_INIT_SPEED

// STRUCTS /////////////////////////////////////

typedef struct typeVehicle
{
	int		state;
	int		type;
	int		dirNextIntersection;
	int		isOkToTurn;
	int		x;
	int		y;
	float	dx;
	float	dy;
	float	ddx;
	float	ddy;
	float	maxDx;
	float	maxDy;
	float	maxDdx;
	float	maxDdy;
	
} vehicle;

// ENUMS ///////////////////////////////////////

enum { GRASS = 0, ROAD, BUILDING, WATER, TILE_TYPE_MAX };
enum { NORTH, EAST, SOUTH, WEST, DIRECTIONS_MAX };
enum { CAR, VEHICLE_TYPE_MAX };
enum { DEAD, ALIVE };

// CONSTANTS /////////////////////////////

const int	SCREEN_WIDTH	= 800;
const int	SCREEN_HEIGHT	= 600;
const int	TILES_X		= 40;
const int	TILES_Y		= 30;
const int	TILE_SIZE	= 20;
const int	MAX_ROADS_TO_REMOVE	= 65;
const int	NUM_INTERSECTIONS_X	= 7;
const int	NUM_INTERSECTIONS_Y	= 5;
const int	MAX_VEHICLES		= 35;
const float	MAX_VEHICLE_SPEED	= 3.0f;
const float	MAX_VEHICLE_ACC		= 0.05f;
const float INIT_VEHICLE_SPEED	= 3.0f;
const float INIT_VEHICLE_ACC	= 0.0f;
const float	CREATE_NEW_VEHICLE_THRESHOLD	= 0.15f;	// 0.05f

// GLOBAL VARS ///////////////////////////////

int		endGame;
float	dt;
int		prevRetraceCount;
BITMAP	*backbuf;
int		numVehicles;
int		nextNewVehicle;

// tiles
char	*tileFileNames[] = { "img/grass.bmp", "img/road.bmp", "img/building.bmp", "img/water.bmp" };
BITMAP	*bmpTiles[TILE_TYPE_MAX];
int		tiles[TILES_Y][TILES_X];

// vehicles
char	*vehicleFileNames[] = { "img/car.bmp" };
BITMAP	*bmpVehicles[VEHICLE_TYPE_MAX];
vehicle	vehicles[MAX_VEHICLES];

// FUNCTIONS ////////////////////////////////

int CreateNewVehicle( int fromDir, int roadXOrY )
{
	if( nextNewVehicle != -1 )
	{
		vehicles[nextNewVehicle].state = ALIVE;
		vehicles[nextNewVehicle].type = CAR;
		vehicles[nextNewVehicle].maxDx = MAX_VEHICLE_SPEED;
		vehicles[nextNewVehicle].maxDy = MAX_VEHICLE_SPEED;
		vehicles[nextNewVehicle].maxDdx = MAX_VEHICLE_ACC;
		vehicles[nextNewVehicle].maxDdy = MAX_VEHICLE_ACC;
		vehicles[nextNewVehicle].isOkToTurn = 0;
		if( fromDir == NORTH )
		{
			//vehicles[nextNewVehicle].dirNextIntersection = SOUTH;
			vehicles[nextNewVehicle].dirNextIntersection = -1;
			vehicles[nextNewVehicle].x = ( ( roadXOrY * 6 ) + 1 ) * TILE_SIZE;
			vehicles[nextNewVehicle].y = -TILE_SIZE;
			vehicles[nextNewVehicle].dx = 0.0f;
#ifdef RAND_VEHICLE_INIT_SPEED
			vehicles[nextNewVehicle].dy = 1.0f + ( ( float )rand() / ( float )RAND_MAX ) * ( MAX_VEHICLE_SPEED - 1.0f );
#else
			vehicles[nextNewVehicle].dy = INIT_VEHICLE_SPEED;
#endif
			vehicles[nextNewVehicle].ddx = 0.0f;
			vehicles[nextNewVehicle].ddy = INIT_VEHICLE_ACC;
		}
		else if( fromDir == SOUTH )
		{
			//vehicles[nextNewVehicle].dirNextIntersection = NORTH;
			vehicles[nextNewVehicle].dirNextIntersection = -1;
			vehicles[nextNewVehicle].x = ( ( roadXOrY * 6 ) + 2 ) * TILE_SIZE;	// "+ 2" so drives in right-hand lane
			vehicles[nextNewVehicle].y = TILES_Y * TILE_SIZE;
			vehicles[nextNewVehicle].dx = 0.0f;
#ifdef RAND_VEHICLE_INIT_SPEED
			vehicles[nextNewVehicle].dy = -1.0f - ( ( float )rand() / ( float )RAND_MAX ) * ( MAX_VEHICLE_SPEED - 1.0f );
#else
			vehicles[nextNewVehicle].dy = -INIT_VEHICLE_SPEED;
#endif
			vehicles[nextNewVehicle].ddx = 0.0f;
			vehicles[nextNewVehicle].ddy = -INIT_VEHICLE_ACC;
		}
		else if( fromDir == EAST )
		{
			//vehicles[nextNewVehicle].dirNextIntersection = WEST;
			vehicles[nextNewVehicle].dirNextIntersection = -1;
			vehicles[nextNewVehicle].x = TILES_X * TILE_SIZE;
			vehicles[nextNewVehicle].y = ( ( roadXOrY * 6 ) + 2 ) * TILE_SIZE;
#ifdef RAND_VEHICLE_INIT_SPEED
			vehicles[nextNewVehicle].dx = -1.0f - ( ( float )rand() / ( float )RAND_MAX ) * ( MAX_VEHICLE_SPEED - 1.0f );
#else
			vehicles[nextNewVehicle].dx = -INIT_VEHICLE_SPEED;
#endif
			vehicles[nextNewVehicle].dy = 0.0f;
			vehicles[nextNewVehicle].ddx = -INIT_VEHICLE_ACC;
			vehicles[nextNewVehicle].ddy = 0.0f;
		}
		else if( fromDir == WEST )
		{
			//vehicles[nextNewVehicle].dirNextIntersection = EAST;
			vehicles[nextNewVehicle].dirNextIntersection = -1;
			vehicles[nextNewVehicle].x = -TILE_SIZE;
			vehicles[nextNewVehicle].y = ( ( roadXOrY * 6 ) + 3 ) * TILE_SIZE;
#ifdef RAND_VEHICLE_INIT_SPEED
			vehicles[nextNewVehicle].dx = 1.0f + ( ( float )rand() / ( float )RAND_MAX ) * ( MAX_VEHICLE_SPEED - 1.0f );
#else
			vehicles[nextNewVehicle].dx = INIT_VEHICLE_SPEED;
#endif
			vehicles[nextNewVehicle].dy = 0.0f;
			vehicles[nextNewVehicle].ddx = INIT_VEHICLE_ACC;
			vehicles[nextNewVehicle].ddy = 0.0f;
		}
		
		numVehicles++;
		
		if( numVehicles < MAX_VEHICLES )
		{
			for( int i = 0; i < MAX_VEHICLES; i++ )
			{
				if( vehicles[i].state == DEAD )
				{
					nextNewVehicle = i;
					break;
				}
			}
		}
		else
		{
			nextNewVehicle = -1;
		}
	}

	return 0;
}

int KillVehicle( int vehicleIndex )
{
	vehicles[vehicleIndex].state = DEAD;
	numVehicles--;

	if( nextNewVehicle == -1 || nextNewVehicle > vehicleIndex )
	{
		nextNewVehicle = vehicleIndex;
	}
	
	return 0;
}

int IsRoadExistsNS( int roadX, int segmentY )
{
	if( roadX < 0 || roadX >= NUM_INTERSECTIONS_X || segmentY < 0 || segmentY > NUM_INTERSECTIONS_Y )
	{
		return 0;	// false
	}

	// to see if road segment exists, test one tile in specified road segment
	int tileX, tileY;
	tileX = ( roadX * 6 ) + 1;
	if( segmentY == NUM_INTERSECTIONS_Y )
	{
		tileY = ( segmentY * 6 ) - 2;
	}
	else
	{
		tileY = ( segmentY * 6 ) + 1;
	}
	
	if( tiles[tileY][tileX] == ROAD )
	{
		return 1;	// true
	}
	else
	{
		return 0;	// false
	}
}

int IsRoadExistsEW( int roadY, int segmentX )
{
	if( roadY < 0 || roadY >= NUM_INTERSECTIONS_Y || segmentX < 0 || segmentX > NUM_INTERSECTIONS_X )
	{
		return 0;	// false
	}

	// to see if road segment exists, test one tile in specified road segment
	int tileX, tileY;
	tileY = ( roadY * 6 ) + 2;
	if( segmentX == NUM_INTERSECTIONS_X )
	{
		tileX = ( segmentX * 6 ) - 3;
	}
	else
	{
		tileX = segmentX * 6;
	}
	
	if( tiles[tileY][tileX] == ROAD )
	{
		return 1;	// true
	}
	else
	{
		return 0;	// false
	}
}

int InitTiles()
{
	int i, x, y;
	
	// load bmp for each tile type
	for( i = 0; i < TILE_TYPE_MAX; i++ )
	{
		bmpTiles[i] = load_bitmap(tileFileNames[i], NULL);
	}

	// randomize tiles!

	// place roads going left to right
	for( y = 0; y < TILES_Y; y++ )
	{
		int temp = ( y - 2 ) % 6;
		if( temp == 0 || temp == 1 )
		{
			for( x = 0; x < TILES_X; x++ )
			{
				tiles[y][x] = ROAD;
			}
		}
	}

	// place roads going up & down
	for( x = 0; x < TILES_X; x++ )
	{
		int temp = ( x - 1 ) % 6;
		if( temp == 0 || temp == 1 )
		{
			for( y = 0; y < TILES_Y; y++ )
			{
				tiles[y][x] = ROAD;
			}
		}
	}

	// remove random sections of road
	int numRoadsRemove = ( (float)rand() / (float)RAND_MAX ) * MAX_ROADS_TO_REMOVE;
	for( i = 0; i < numRoadsRemove; i++ )
	{
		// randomly pick an intersection
		int randX = ( (float)rand() / (float)RAND_MAX ) * NUM_INTERSECTIONS_X;
		int randY = ( (float)rand() / (float)RAND_MAX ) * NUM_INTERSECTIONS_Y;
		//allegro_message( "randX = %d, randY = %d", randX, randY );

		// randomly pick a direction
		int randDir = ( (float)rand() / (float)RAND_MAX ) * DIRECTIONS_MAX;
		
		// remove section of road
		int removeX;
		int removeY;
		if( randDir == NORTH )
		{
			// find upper-left corner of 2 col X 4 row section of road above this intersection to remove
			removeX = ( randX * 6 ) + 1;
			removeY = ( randY * 6 ) - 2;
			for( y = removeY; y < removeY + 4; y++ )
			{
				for( x = removeX; x < removeX + 2; x++ )
				{
					if( y >= 0 && y < TILES_Y && x >= 0 && x < TILES_X )
					{
						tiles[y][x] = GRASS;
					}
				}
			}
		}
		else if( randDir == SOUTH )
		{
			// find upper-left corner of 2 col X 4 row section of road below this intersection to remove
			removeX = ( randX * 6 ) + 1;
			removeY = ( randY * 6 ) + 4;
			for( y = removeY; y < removeY + 4; y++ )
			{
				for( x = removeX; x < removeX + 2; x++ )
				{
					if( y >= 0 && y < TILES_Y && x >= 0 && x < TILES_X )
					{
						tiles[y][x] = GRASS;
					}
				}
			}
		}
		else if( randDir == WEST )
		{
			// find upper-left corner of 4 col X 2 row section of road to the left of this intersection to remove
			removeX = ( randX * 6 ) - 3;
			removeY = ( randY * 6 ) + 2;
			for( y = removeY; y < removeY + 2; y++ )
			{
				for( x = removeX; x < removeX + 4; x++ )
				{
					if( y >= 0 && y < TILES_Y && x >= 0 && x < TILES_X )
					{
						tiles[y][x] = GRASS;
					}
				}
			}
		}
		else if( randDir == EAST )
		{
			// find upper-left corner of 4 col X 2 row section of road to the right of this intersection to remove
			removeX = ( randX * 6 ) + 3;
			removeY = ( randY * 6 ) + 2;
			for( y = removeY; y < removeY + 2; y++ )
			{
				for( x = removeX; x < removeX + 4; x++ )
				{
					if( y >= 0 && y < TILES_Y && x >= 0 && x < TILES_X )
					{
						tiles[y][x] = GRASS;
					}
				}
			}
		}		
	}

	return 0;
}

int InitVehicles()
{
	int i;

	numVehicles = 0;
	nextNewVehicle = 0;

	// load the bmp for each vehicle type
	for( i = 0; i < VEHICLE_TYPE_MAX; i++ )
	{
		bmpVehicles[i] = load_bitmap( vehicleFileNames[i], NULL );
	}

	for( i = 0; i < MAX_VEHICLES; i++ )
	{
		vehicles[i].state = DEAD;
	}
	
	return 0;
}

int Init()
{
	// init allegro
	allegro_init();
	install_timer();
	text_mode(-1);
	install_keyboard();
	if(install_mouse() == 0)
		return -1;
	set_color_depth( 16 );
	if( set_gfx_mode( GFX_AUTODETECT_WINDOWED, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0 ) )
		return -2;

	// load bitmaps
	
	backbuf = create_bitmap( SCREEN_WIDTH, SCREEN_HEIGHT );

	InitTiles();
	InitVehicles();

	endGame = 0;
	dt = 0.0f;
	prevRetraceCount = retrace_count;

	return 0;
}

int UnInit()
{
	return 0;
}

int Draw()
{
	// tiles
	for( int y = 0; y < TILES_Y; y++ )
	{
		for( int x = 0; x < TILES_X; x++ )
		{
			blit( bmpTiles[tiles[y][x]], backbuf, 0, 0, x * TILE_SIZE, y * TILE_SIZE, TILE_SIZE, TILE_SIZE );
		}
	}

	// vehicles
	for( int i = 0; i < MAX_VEHICLES; i++ )
	{
		if( vehicles[i].state == ALIVE )
		{
			masked_blit( bmpVehicles[vehicles[i].type], backbuf, 0, 0, vehicles[i].x, vehicles[i].y, TILE_SIZE, TILE_SIZE );
		}
	}

	//textout( backbuf, font, "blah", 50, 50, 2 );
	//_RPT0{CRT_WARN "hello world"};

	// flip
	vsync();
	acquire_screen();
	blit( backbuf, screen, 0, 0, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT );
	release_screen();
	yield_timeslice();
	
	return 0;
}

int VehicleLifeCycle()
{
	// create new vehicle?
	if( numVehicles < MAX_VEHICLES )
	{
		float shouldCreateNewVehicle = (float)rand() / (float)RAND_MAX;
		if( shouldCreateNewVehicle < CREATE_NEW_VEHICLE_THRESHOLD )
		{
			// pick an entry point on map
			int fromDir = ( (float)rand() / (float)RAND_MAX ) * DIRECTIONS_MAX;
			if( fromDir == NORTH )
			{
				int roadX = ( (float)rand() / (float)RAND_MAX ) * NUM_INTERSECTIONS_X;
				if( IsRoadExistsNS( roadX, 0 ) )
				{
					CreateNewVehicle( NORTH, roadX );
				}
			}
			else if( fromDir == SOUTH )
			{
				int roadX = ( (float)rand() / (float)RAND_MAX ) * NUM_INTERSECTIONS_X;
				if( IsRoadExistsNS( roadX, NUM_INTERSECTIONS_Y ) )
				{
					CreateNewVehicle( SOUTH, roadX );
				}
			}
			else if( fromDir == EAST )
			{
				int roadY = ( (float)rand() / (float)RAND_MAX ) * NUM_INTERSECTIONS_Y;
				if( IsRoadExistsEW( roadY, NUM_INTERSECTIONS_X ) )
				{
					CreateNewVehicle( EAST, roadY );
				}
			}
			else if( fromDir == WEST )
			{
				int roadY = ( (float)rand() / (float)RAND_MAX ) * NUM_INTERSECTIONS_Y;
				if( IsRoadExistsEW( roadY, 0 ) )
				{
					CreateNewVehicle( WEST, roadY );
				}
			}
		}
	}
	
	return 0;
}

int ProcessInput()
{
	if( key[KEY_ESC] )
	{
		endGame = 1;
	}
	
	if( key[KEY_UP] )
	{
		vehicles[0].dirNextIntersection = NORTH;
	}
	else if( key[KEY_DOWN] )
	{
		vehicles[0].dirNextIntersection = SOUTH;
	}
	else if( key[KEY_LEFT] )
	{
		vehicles[0].dirNextIntersection = WEST;
	}
	else if( key[KEY_RIGHT] )
	{
		vehicles[0].dirNextIntersection = EAST;
	}

	return 0;
}

int RandomVehicleDir( int vehicleIndex )
{
	int roadNSX, segNSY, roadEWY, segEWX;
	int isOk[DIRECTIONS_MAX];
	int numOkRoads = 0;

	// check if n and s roads exist
	//int temp = ( vehicles[vehicleIndex].x / TILE_SIZE ) - 5;
	//if( temp < 0 )
	//	temp = 0;
	roadNSX = ( ( vehicles[vehicleIndex].x / TILE_SIZE ) + 1 ) / 6;
	//roadNSX = temp / 6;
	//temp = ( vehicles[vehicleIndex].y / TILE_SIZE ) - 0;
	//if( temp < 0 )
	//	temp = 0;
	segNSY = ( vehicles[vehicleIndex].y / TILE_SIZE / 6 );
	//segNSY = temp / 6;
	if( IsRoadExistsNS( roadNSX, segNSY ) )
	{
		isOk[NORTH] = 1;
		numOkRoads++;
	}
	else
	{
		isOk[NORTH] = 0;
	}
	if( IsRoadExistsNS( roadNSX, segNSY + 1 ) )
	{
		isOk[SOUTH] = 1;
		numOkRoads++;
	}
	else
	{
		isOk[SOUTH] = 0;
	}

	// check if e and w roads exist
	//temp = ( vehicles[vehicleIndex].y / TILE_SIZE ) - 0;
	//if( temp < 0 )
	//	temp = 0;
	roadEWY = ( vehicles[vehicleIndex].y / TILE_SIZE / 6 );
	//roadEWY = temp / 6;
	//temp = ( vehicles[vehicleIndex].x / TILE_SIZE ) - 5;
	//if( temp < 0 )
	//	temp = 0;
	segEWX = ( ( vehicles[vehicleIndex].x / TILE_SIZE ) + 1 ) / 6;
	//segEWX = temp / 6;
	if( IsRoadExistsEW( roadEWY, segEWX ) )
	{
		isOk[WEST] = 1;
		numOkRoads++;
	}
	else
	{
		isOk[WEST] = 0;
	}
	if( IsRoadExistsEW( roadEWY, segEWX + 1 ) )
	{
		isOk[EAST] = 1;
		numOkRoads++;
	}
	else
	{
		isOk[EAST] = 0;
	}

	// get dir we're travelling in
	int curFromDir;
	if( vehicles[vehicleIndex].dx > 0 )
		curFromDir = WEST;
	if( vehicles[vehicleIndex].dx < 0 )
		curFromDir = EAST;
	if( vehicles[vehicleIndex].dy > 0 )
		curFromDir = NORTH;
	if( vehicles[vehicleIndex].dy < 0 )
		curFromDir = SOUTH;

	// choose a road at this intersection
	if( numOkRoads == 1 )
	{
		// u-turn
		if( curFromDir == EAST )
			vehicles[vehicleIndex].dirNextIntersection = EAST;
		else if( curFromDir == WEST )
			vehicles[vehicleIndex].dirNextIntersection = WEST;
		else if( curFromDir == SOUTH )
			vehicles[vehicleIndex].dirNextIntersection = SOUTH;
		else
			vehicles[vehicleIndex].dirNextIntersection = NORTH;
	}
	else
	{
		// choose a dir that's not a u-turn
		/*for( int i = 0; i < DIRECTIONS_MAX; i++ )
		{
			if( isOk[i] && curFromDir != i )
			{
				vehicles[vehicleIndex].dirNextIntersection = i;
				break;
			}
		}*/

		float dir = ( ( float )rand() / ( float )RAND_MAX );
		int isChosen = 0;
		// straight
		if( dir < 0.8f )
		{
			if( isOk[SOUTH] && curFromDir == NORTH )
			{
				vehicles[vehicleIndex].dirNextIntersection = SOUTH;
				isChosen = 1;
			}
			if( isOk[WEST] && curFromDir == EAST )
			{
				vehicles[vehicleIndex].dirNextIntersection = WEST;
				isChosen = 1;
			}
			if( isOk[NORTH] && curFromDir == SOUTH )
			{
				vehicles[vehicleIndex].dirNextIntersection = NORTH;
				isChosen = 1;
			}
			if( isOk[EAST] && curFromDir == WEST )
			{
				vehicles[vehicleIndex].dirNextIntersection = EAST;
				isChosen = 1;
			}
		}
		// left
		else if( dir < 0.9f )
		{
			if( isOk[EAST] && curFromDir == NORTH )
			{
				vehicles[vehicleIndex].dirNextIntersection = EAST;
				isChosen = 1;
			}
			if( isOk[SOUTH] && curFromDir == EAST )
			{
				vehicles[vehicleIndex].dirNextIntersection = SOUTH;
				isChosen = 1;
			}
			if( isOk[WEST] && curFromDir == SOUTH )
			{
				vehicles[vehicleIndex].dirNextIntersection = WEST;
				isChosen = 1;
			}
			if( isOk[NORTH] && curFromDir == WEST )
			{
				vehicles[vehicleIndex].dirNextIntersection = NORTH;
				isChosen = 1;
			}
		}
		// right
		else
		{
			if( isOk[WEST] && curFromDir == NORTH )
			{
				vehicles[vehicleIndex].dirNextIntersection = WEST;
				isChosen = 1;
			}
			if( isOk[NORTH] && curFromDir == EAST )
			{
				vehicles[vehicleIndex].dirNextIntersection = NORTH;
				isChosen = 1;
			}
			if( isOk[EAST] && curFromDir == SOUTH )
			{
				vehicles[vehicleIndex].dirNextIntersection = EAST;
				isChosen = 1;
			}
			if( isOk[SOUTH] && curFromDir == WEST )
			{
				vehicles[vehicleIndex].dirNextIntersection = SOUTH;
				isChosen = 1;
			}
		}

		// if still no match then assign one
		if( !isChosen )
		{
			for( int i = 0; i < DIRECTIONS_MAX; i++ )
			{
				if( isOk[i] && curFromDir != i )
				{
					vehicles[vehicleIndex].dirNextIntersection = i;
					break;
				}
			}
		}
	}

	return 0;
}

int DoIntersectionMovement( int vehicleIndex )
{
	// change direction

	// moving south
	if( vehicles[vehicleIndex].dy > 0.0f )
	{
		// at 1st tile of intersection?
		if( ( vehicles[vehicleIndex].y / TILE_SIZE ) % 6 == 2 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				//if( vehicles[vehicleIndex].dirNextIntersection < 0 )
				//{
					//vehicles[vehicleIndex].isOkToTurn = 1;
					RandomVehicleDir( vehicleIndex );
				//}

				if( vehicles[vehicleIndex].dirNextIntersection == WEST )
				{
					vehicles[vehicleIndex].y = ( vehicles[vehicleIndex].y / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dx = -vehicles[vehicleIndex].dy;
					vehicles[vehicleIndex].dy = 0.0f;
					vehicles[vehicleIndex].ddx = -vehicles[vehicleIndex].ddy;
					vehicles[vehicleIndex].ddy = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		// at 2nd tile of intersection?
		else if( ( vehicles[vehicleIndex].y / TILE_SIZE ) % 6 == 3 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				if( vehicles[vehicleIndex].dirNextIntersection == EAST )
				{
					vehicles[vehicleIndex].y = ( vehicles[vehicleIndex].y / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dx = vehicles[vehicleIndex].dy;
					vehicles[vehicleIndex].dy = 0.0f;
					vehicles[vehicleIndex].ddx = vehicles[vehicleIndex].ddy;
					vehicles[vehicleIndex].ddy = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
				else if( vehicles[vehicleIndex].dirNextIntersection == NORTH )
				{
					vehicles[vehicleIndex].y = ( vehicles[vehicleIndex].y / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].x += TILE_SIZE;
					vehicles[vehicleIndex].dy = -vehicles[vehicleIndex].dy;
					vehicles[vehicleIndex].ddy = -vehicles[vehicleIndex].ddy;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		else
		{
			vehicles[vehicleIndex].isOkToTurn = 1;
		}
	}
	// moving north
	else if( vehicles[vehicleIndex].dy < 0.0f )
	{
		// at 1st tile of intersection?
		if( ( vehicles[vehicleIndex].y / TILE_SIZE ) % 6 == 3 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				//if( vehicles[vehicleIndex].dirNextIntersection < 0 )
				//{
					//vehicles[vehicleIndex].isOkToTurn = 1;
					RandomVehicleDir( vehicleIndex );
				//}

				if( vehicles[vehicleIndex].dirNextIntersection == EAST )
				{
					vehicles[vehicleIndex].y = ( vehicles[vehicleIndex].y / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dx = -vehicles[vehicleIndex].dy;
					vehicles[vehicleIndex].dy = 0.0f;
					vehicles[vehicleIndex].ddx = -vehicles[vehicleIndex].ddy;
					vehicles[vehicleIndex].ddy = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		// at 2nd tile of intersection?
		else if( ( vehicles[vehicleIndex].y / TILE_SIZE ) % 6 == 2 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				if( vehicles[vehicleIndex].dirNextIntersection == WEST )
				{
					vehicles[vehicleIndex].y = ( vehicles[vehicleIndex].y / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dx = vehicles[vehicleIndex].dy;
					vehicles[vehicleIndex].dy = 0.0f;
					vehicles[vehicleIndex].ddx = vehicles[vehicleIndex].ddy;
					vehicles[vehicleIndex].ddy = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
				else if( vehicles[vehicleIndex].dirNextIntersection == SOUTH )
				{
					vehicles[vehicleIndex].y = ( vehicles[vehicleIndex].y / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].x -= TILE_SIZE;
					vehicles[vehicleIndex].dy = -vehicles[vehicleIndex].dy;
					vehicles[vehicleIndex].ddy = -vehicles[vehicleIndex].ddy;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		else
		{
			vehicles[vehicleIndex].isOkToTurn = 1;
		}
	}
	// moving west
	else if( vehicles[vehicleIndex].dx < 0.0f )
	{
		// at 1st tile of intersection?
		if( ( vehicles[vehicleIndex].x / TILE_SIZE ) % 6 == 2 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				//if( vehicles[vehicleIndex].dirNextIntersection < 0 )
				//{
					//vehicles[vehicleIndex].isOkToTurn = 1;
					RandomVehicleDir( vehicleIndex );
				//}

				if( vehicles[vehicleIndex].dirNextIntersection == NORTH )
				{
					vehicles[vehicleIndex].x = ( vehicles[vehicleIndex].x / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dy = vehicles[vehicleIndex].dx;
					vehicles[vehicleIndex].dx = 0.0f;
					vehicles[vehicleIndex].ddy = vehicles[vehicleIndex].ddx;
					vehicles[vehicleIndex].ddx = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		// at 2nd tile of intersection?
		else if( ( vehicles[vehicleIndex].x / TILE_SIZE ) % 6 == 1 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				if( vehicles[vehicleIndex].dirNextIntersection == SOUTH )
				{
					vehicles[vehicleIndex].x = ( vehicles[vehicleIndex].x / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dy = -vehicles[vehicleIndex].dx;
					vehicles[vehicleIndex].dx = 0.0f;
					vehicles[vehicleIndex].ddy = -vehicles[vehicleIndex].ddx;
					vehicles[vehicleIndex].ddx = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
				else if( vehicles[vehicleIndex].dirNextIntersection == EAST )
				{
					vehicles[vehicleIndex].x = ( vehicles[vehicleIndex].x / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].y += TILE_SIZE;
					vehicles[vehicleIndex].dx = -vehicles[vehicleIndex].dx;
					vehicles[vehicleIndex].ddx = -vehicles[vehicleIndex].ddx;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		else
		{
			vehicles[vehicleIndex].isOkToTurn = 1;
		}
	}
	// moving east
	else if( vehicles[vehicleIndex].dx > 0.0f )
	{
		// at 1st tile of intersection?
		if( ( vehicles[vehicleIndex].x / TILE_SIZE ) % 6 == 1 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				//if( vehicles[vehicleIndex].dirNextIntersection < 0 )
				//{
					//vehicles[vehicleIndex].isOkToTurn = 1;
					RandomVehicleDir( vehicleIndex );
				//}

				if( vehicles[vehicleIndex].dirNextIntersection == SOUTH )
				{
					vehicles[vehicleIndex].x = ( vehicles[vehicleIndex].x / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dy = vehicles[vehicleIndex].dx;
					vehicles[vehicleIndex].dx = 0.0f;
					vehicles[vehicleIndex].ddy = vehicles[vehicleIndex].ddx;
					vehicles[vehicleIndex].ddx = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		// at 2nd tile of intersection?
		else if( ( vehicles[vehicleIndex].x / TILE_SIZE ) % 6 == 2 )
		{
			if( vehicles[vehicleIndex].isOkToTurn )
			{
				if( vehicles[vehicleIndex].dirNextIntersection == NORTH )
				{
					vehicles[vehicleIndex].x = ( vehicles[vehicleIndex].x / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].dy = -vehicles[vehicleIndex].dx;
					vehicles[vehicleIndex].dx = 0.0f;
					vehicles[vehicleIndex].ddy = -vehicles[vehicleIndex].ddx;
					vehicles[vehicleIndex].ddx = 0.0f;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
				else if( vehicles[vehicleIndex].dirNextIntersection == WEST )
				{
					vehicles[vehicleIndex].x = ( vehicles[vehicleIndex].x / TILE_SIZE ) * TILE_SIZE;
					vehicles[vehicleIndex].y -= TILE_SIZE;
					vehicles[vehicleIndex].dx = -vehicles[vehicleIndex].dx;
					vehicles[vehicleIndex].ddx = -vehicles[vehicleIndex].ddx;

					vehicles[vehicleIndex].isOkToTurn = 0;
					vehicles[vehicleIndex].dirNextIntersection = -1;
				}
			}
		}
		else
		{
			vehicles[vehicleIndex].isOkToTurn = 1;
		}
	}

	return 0;
}

int UpdateGameObjects()
{
	int i;

	dt = ( ( float )( retrace_count - prevRetraceCount ) )/* / 70.0f*/;
	prevRetraceCount = retrace_count;

	for( i = 0; i < MAX_VEHICLES; i++ )
	{
		if( vehicles[i].state == ALIVE )
		{
			// if at intersection, turn left, right, or straight
			DoIntersectionMovement(i);

			// update acceleration
			/*if( vehicles[i].ddx > MAX_VEHICLE_ACC )
				vehicles[i].ddx = MAX_VEHICLE_ACC;
			if( vehicles[i].ddx < -MAX_VEHICLE_ACC )
				vehicles[i].ddx = -MAX_VEHICLE_ACC;
			if( vehicles[i].ddy > MAX_VEHICLE_ACC )
				vehicles[i].ddy = MAX_VEHICLE_ACC;
			if( vehicles[i].ddy < -MAX_VEHICLE_ACC )
				vehicles[i].ddy = -MAX_VEHICLE_ACC;*/
			//vehicles[i].ddx = 0.0f;
			//vehicles[i].ddy = 0.0f;

			// update velocity
			vehicles[i].dx += vehicles[i].ddx * dt;
			vehicles[i].dy += vehicles[i].ddy * dt;
			if( vehicles[i].dx > MAX_VEHICLE_SPEED )
				vehicles[i].dx = MAX_VEHICLE_SPEED;
			if( vehicles[i].dx < -MAX_VEHICLE_SPEED )
				vehicles[i].dx = -MAX_VEHICLE_SPEED;
			if( vehicles[i].dy > MAX_VEHICLE_SPEED )
				vehicles[i].dy = MAX_VEHICLE_SPEED;
			if( vehicles[i].dy < -MAX_VEHICLE_SPEED )
				vehicles[i].dy = -MAX_VEHICLE_SPEED;

			// update pos
			vehicles[i].x += vehicles[i].dx * dt;
			vehicles[i].y += vehicles[i].dy * dt;

			// slow down if going to hit someone
			for( int j = 0; j < MAX_VEHICLES; j++ )
			{
				if( vehicles[j].state == ALIVE && i != j )
				{
					// east-west
					if( vehicles[i].y == vehicles[j].y && vehicles[i].dy == 0 && vehicles[j].dy == 0 &&	// moving in same dir
						(	( vehicles[i].dx > 0 && vehicles[i].x < vehicles[j].x && vehicles[j].x - vehicles[i].x < TILE_SIZE ) ||	// this one's behind, may need to slow down
							( vehicles[i].dx < 0 && vehicles[i].x > vehicles[j].x && vehicles[i].x - vehicles[j].x < TILE_SIZE )		// this one's behind, may need to slow down
						) )
					{
						vehicles[i].dx = vehicles[j].dx;
						if( vehicles[i].dx > 0 )
						{
							vehicles[i].x = vehicles[j].x - TILE_SIZE;
						}
						else
						{
							vehicles[i].x = vehicles[j].x + TILE_SIZE;
						}
					}
					// north-south
					if( vehicles[i].x == vehicles[j].x && vehicles[i].dx == 0 && vehicles[j].dx == 0 &&	// moving in same dir
						(	( vehicles[i].dy > 0 && vehicles[i].y < vehicles[j].y && vehicles[j].y - vehicles[i].y < TILE_SIZE ) ||	// this one's behind, may need to slow down
							( vehicles[i].dy < 0 && vehicles[i].y > vehicles[j].y && vehicles[i].y - vehicles[j].y < TILE_SIZE )		// this one's behind, may need to slow down
						) )
					{
						vehicles[i].dy = vehicles[j].dy;
						if( vehicles[i].dy > 0 )
						{
							vehicles[i].y = vehicles[j].y - TILE_SIZE;
						}
						else
						{
							vehicles[i].y = vehicles[j].y + TILE_SIZE;
						}
					}
				}
			}

			// kill vehicles that have gone off-screen
			// make sure to leave an extra tile width/height around outside so don't delete
			// vehicles that have just been born there & are about to appear on screen
			if( vehicles[i].x < ( -2 * TILE_SIZE ) || vehicles[i].x > ( TILES_X + 1 ) * TILE_SIZE ||
				vehicles[i].y < ( -2 * TILE_SIZE ) || vehicles[i].y > ( TILES_Y + 1 ) * TILE_SIZE )
			{
				KillVehicle( i );
			}
		}
	}
	
	return 0;
}

int main( int argc, char* argv[] )
{
	// assume successful until error occurs
	int isSuccess = 1;

	srand( time( 0 ) );

	Init();

	// main game loop
	do
	{
		VehicleLifeCycle();
		ProcessInput();
		UpdateGameObjects();
		Draw();
	}
	while(endGame == 0);

	UnInit();

	return 0;
}
END_OF_MAIN();